@@ -15,8 +15,9 @@ from . import func
 
 from .function_parser.custom_wrapper import controlled_wrapper, update_wrapper
 from .function_parser import serialize_type
+from .function_parser.types import add_type
 
-__version__ = "0.3.4"
+__version__ = "0.3.7"
 
 __all__ = [
     "function_parser",
@@ -36,4 +37,5 @@ __all__ = [
     "controlled_wrapper",
     "update_wrapper",
     "serialize_type",
+    "add_type",
 ]
