@@ -10,7 +10,6 @@ from typing import get_type_hints
 from .types import (
     SerializedFunction,
     DocstringParserResult,
-    FunctionParamError,
     FunctionInputParam,
     type_to_string,
     Optional,
@@ -94,7 +93,7 @@ def get_resolved_signature(
     # from any nested partials.
 
     # Obtain the original signature
-    sig = inspect.signature(base_func)
+    sig = inspect.signature(base_func, follow_wrapped=False)
 
     params = list(sig.parameters.values())
 
@@ -181,12 +180,20 @@ def function_method_parser(
         sig, base_func = get_resolved_signature(func)
 
         for i, p in sig.parameters.items():
+            n = i
+            if p.kind == p.VAR_POSITIONAL:
+                n = "*" + n
+            if p.kind == p.VAR_KEYWORD:
+                n = "**" + n
+
             param_dict: FunctionInputParam = {
-                "name": i,
+                "name": n,
                 "default": p.default,
                 "type": th[i] if i in th else p.empty,
                 "positional": (
-                    p.kind == p.POSITIONAL_ONLY or p.kind == p.POSITIONAL_OR_KEYWORD
+                    p.kind == p.POSITIONAL_ONLY
+                    or p.kind == p.POSITIONAL_OR_KEYWORD
+                    or p.kind == p.VAR_POSITIONAL
                 )
                 and (p.default == p.empty),
             }
@@ -201,9 +208,13 @@ def function_method_parser(
                 try:
                     json.dumps(param_dict["default"])
                 except TypeError as exe:
-                    raise FunctionParamError(
-                        f"input {i} has unserializable default value '{param_dict['default']}'"
-                    ) from exe
+                    try:
+                        param_dict["default"] = param_dict["default"].__name__
+                    except AttributeError:
+                        param_dict["default"] = str(param_dict["default"])
+                        # raise FunctionParamError(
+                        #     f"input {i} has unserializable default value '{param_dict['default']}'"
+                        # ) from exe
             else:
                 del param_dict["default"]
 
@@ -228,7 +239,6 @@ def function_method_parser(
         else:
             output_params = [{"name": "out", "type": type_to_string(th["return"])}]
 
-
     if parsed_ds is not None:
         # update input params
         for p in input_params:
@@ -236,18 +246,17 @@ def function_method_parser(
                 if p["name"] != parsed_ip["name"]:
                     continue
 
-                if (
-                    "description" not in p or p["description"] is None
-                ) and "description" in parsed_ip:
+                if ("description" not in p) and "description" in parsed_ip:
                     p["description"] = parsed_ip["description"]
-                if (
-                    "optional" not in p or p["optional"] is None
-                ) and "optional" in parsed_ip:
-                    p["optional"] = parsed_ip["optional"]
+                # optinoanl should be set by the parser
+                # if ("optional" not in p) and "optional" in parsed_ip:
+                #     p["optional"] = parsed_ip["optional"]
 
                 if (
-                    "default" not in p or p["default"] is None
-                ) and "default" in parsed_ip:
+                    ("default" not in p)
+                    and "default" in parsed_ip
+                    and p.get("optional", False)
+                ):
                     p["default"] = parsed_ip["default"]
 
                 if (
@@ -258,6 +267,13 @@ def function_method_parser(
                 ) and "type" in parsed_ip:
                     p["type"] = parsed_ip["type"]
 
+                # a default value makes the parameter optional by default and the parameter non-positional
+                if "default" in p:
+                    p["optional"] = True
+                    p["positional"] = False
+                else:
+                    p["optional"] = False
+                    p["positional"] = True
                 # possitional is always set
                 # if (
                 #    "positional" not in p or p["positional"] is None
