@@ -1,4 +1,5 @@
 from __future__ import annotations
+from enum import Enum
 import importlib
 import sys
 import re
@@ -17,6 +18,8 @@ from typing import (
     Protocol,
     TypeVar,
     Sequence,
+    get_origin,
+    get_args,
 )
 import collections
 
@@ -398,7 +401,7 @@ def type_to_string(t: Union[type, str]):
             elif origin in [tuple, Tuple]:
                 return f"Tuple[{', '.join([type_to_string(subtype) for subtype in t.__args__])}]"
             elif origin is Union:
-                return f"Union[{', '.join(sorted([type_to_string(subtype) for subtype in t.__args__]))}]"
+                return f"Union[{', '.join([type_to_string(subtype) for subtype in t.__args__])}]"
             elif origin in [Type, type]:
                 if hasattr(t, "__args__"):
                     return f"Type[{type_to_string(t.__args__[0])}]"
@@ -449,11 +452,13 @@ def cast_to_type(value: Any, type_):
     except Exception as e:
         pass
 
-    origin = getattr(type_, "__origin__", None)
+    origin = get_origin(type_)
+
     ex = []
     if origin:
+        args = get_args(type_)
         if origin is Union:
-            for subtype in type_.__args__:
+            for subtype in args:
                 try:
                     return cast_to_type(value, subtype)
                 except Exception as e:
@@ -462,7 +467,7 @@ def cast_to_type(value: Any, type_):
             if value in (None, "", "None", "none"):
                 return None
             else:
-                return cast_to_type(value, type_.__args__[0])
+                return cast_to_type(value, args[0])
 
     ex.append(ValueError(f"Could not cast {value} to type {type_}"))
 
@@ -474,3 +479,126 @@ def cast_to_type(value: Any, type_):
         except Exception as ne:
             e = ne
     raise e
+
+
+class AllOf(TypedDict):
+    allOf: List[SerializedType]
+
+
+class AnyOf(TypedDict):
+    anyOf: List[SerializedType]
+
+
+class ArrayOf(TypedDict):
+    type: Literal["array"]
+    items: SerializedType
+    uniqueItems: bool
+
+
+class DictOf(TypedDict):
+    type: Literal["object"]
+    keys: SerializedType
+    values: SerializedType
+
+
+class EnumOf(TypedDict):
+    type: Literal["enum"]
+    values: List[Union[int, float, str, bool, None]]
+    keys: List[str]
+    nullable: bool
+
+
+class TypeOf(TypedDict):
+    type: Literal["type"]
+    value: SerializedType
+
+
+SerializedType = Union[str, AllOf, AnyOf, ArrayOf, DictOf, EnumOf, TypeOf]
+
+
+def serialize_type(type_: type) -> SerializedType:
+
+    origin = get_origin(type_)
+    args = get_args(type_)
+
+    if origin is Union:
+        hasNone = False
+        if None in args or NoneType in args:
+            hasNone = True
+            args = [arg for arg in args if arg is not None and arg is not NoneType]
+
+        subtypes = [serialize_type(subtype) for subtype in args]
+
+        if hasNone:
+            for st in subtypes:
+                # make enums nullable
+                if isinstance(st, dict) and "type" in st:
+                    if st["type"] == "enum":
+                        st["nullable"] = True
+                        if len(subtypes) == 1:
+                            return st
+
+            nonestr = type_to_string(NoneType)
+            if nonestr not in subtypes:
+                subtypes.append(nonestr)
+        if len(subtypes) == 1:
+            return subtypes[0]
+        seen = set()
+        seen_add = seen.add
+        return AnyOf(
+            anyOf=[x for x in subtypes if not (str(x) in seen or seen_add(str(x)))]
+        )
+
+    elif origin is Optional:
+        return serialize_type(Union[args + (None,)])
+    elif origin in [tuple, Tuple]:
+        return AllOf(
+            allOf=[serialize_type(subtype) for subtype in args],
+        )
+
+    elif origin in [List, list, Sequence, collections.abc.Sequence]:
+        return ArrayOf(
+            type="array",
+            uniqueItems=False,
+            items=serialize_type(args[0]),
+        )
+    elif origin in [Dict, dict]:
+        return DictOf(
+            type="object",
+            keys=serialize_type(args[0]),
+            values=serialize_type(args[1]),
+        )
+    elif origin is Literal:
+        typestrings = [item for item in args]
+        nullable = False
+        if None in typestrings:
+            typestrings.remove(None)
+            nullable = True
+        return EnumOf(
+            type="enum",
+            values=typestrings,
+            keys=[str(item) for item in typestrings],
+            nullable=nullable,
+        )
+
+    elif origin in [Set, set]:
+        return ArrayOf(
+            type="array",
+            uniqueItems=True,
+            items=serialize_type(args[0]),
+        )
+    elif origin in [Type, type]:
+        return TypeOf(
+            type="type",
+            value=serialize_type(args[0] if args else Any),
+        )
+
+    if (isinstance(type_, type) and issubclass(type_, Enum)) or isinstance(type_, Enum):
+        return EnumOf(
+            type="enum",
+            values=[member.value for member in type_],
+            keys=[member.name for member in type_],
+            nullable=False,
+        )
+
+    return type_to_string(type_)
